/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of FRETSaw.
 *
 *    FRETSaw is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FRETSaw is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FretSaw;

import ij.ImagePlus;
import ij.io.FileSaver;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;

import uk.ac.bbsrc.babraham.FretSaw.Display.FretSawMenuBar;
import uk.ac.bbsrc.babraham.FretSaw.Display.ImageSetPanel;
import uk.ac.bbsrc.babraham.FretSaw.IO.ImageLoader;
import uk.ac.bbsrc.babraham.FretSaw.IO.ImageSet;
import uk.ac.bbsrc.babraham.FretSaw.Preferences.FretSawPreferences;

public class FretSawApplication extends JFrame {

	public static final String VERSION = "1.0";
	
	private ImageSetPanel images;
	private ImageSet imageSet = null;
	
	private static Vector<FretSawApplication> apps = new Vector<FretSawApplication>();
	
	public FretSawApplication () {
		
		super("FRET Saw");
		setIconImage(new ImageIcon(ClassLoader.getSystemResource("Resources/fretsaw_logo.png")).getImage());
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setSize(1024, 400);
		setLocationRelativeTo(null);
		setLocation(getLocation().x+(10*apps.size()),getLocation().y+(10*apps.size()));
		
		setJMenuBar(new FretSawMenuBar(this));
		images = new ImageSetPanel();
		setContentPane(images);
		
		setVisible(true);
		
		apps.add(this);
	}
		
	public void openImage (ImageLoader loader) {
		
		ImageSet newSet;
		try {
			newSet = loader.getImageSet();
		} 
		catch (IOException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(),"Error loading image",JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (newSet != null) {
			// Load these images and create a new fret image
			setTitle("FRET Saw ["+loader.getFileName()+"]");
			FretSawPreferences.getInstance().setLastDirectory(loader.getLastDirectory());
			imageSet = newSet;
			images.setImages(imageSet);
		}
		
						
	}
	
	public void dispose () {
		apps.remove(this);
		if (apps.size() == 0) System.exit(0);
		super.dispose();
	}
	
	
	public void saveImage () {

		ImagePlus fretImage = images.getFretImage();
		if (fretImage == null) {
			JOptionPane.showMessageDialog(this, "No FRET Image is available", "Can't save image", JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		JFileChooser chooser = new JFileChooser(FretSawPreferences.getInstance().lastDirectory());
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "TIFF Files";
			}
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".tif") || f.getName().toLowerCase().endsWith(".tiff")) {
					return true;
				}
				return false;
			}
		});
		
		
		int result = chooser.showSaveDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) return;

		File file = chooser.getSelectedFile();
		
		if (file != null) {
			
			if (!(file.getName().toLowerCase().endsWith(".tif") || file.getName().toLowerCase().endsWith(".tiff"))) {
				file = new File(file.getAbsolutePath()+".tif");
			}
			
			FretSawPreferences.getInstance().setLastDirectory(file);
			FileSaver fs = new FileSaver(images.getFretImage());			
			fs.saveAsTiffStack(file.getAbsolutePath());
		}		
		
	}
	
	public void updateImage () {
		images.update();
	}		
	
	public static void main(String[] args) {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}
		
		new FretSawApplication();
	}

}
