/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of FRETSaw.
 *
 *    FRETSaw is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FRETSaw is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FretSaw.Preferences;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import uk.ac.bbsrc.babraham.FretSaw.FretSawApplication;

public class PreferencesEditor extends JDialog implements ActionListener {

	FretSawApplication application;
	JTextField minIntensity;
	JTextField minRatio;
	JTextField maxRatio;
	JTextField smoothing;
	JTextField brightness;
	JCheckBox overlay;
	
	public PreferencesEditor (FretSawApplication application) {
		super(application,"Edit Preferences");
		this.application = application;
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		
		FretSawPreferences prefs = FretSawPreferences.getInstance();
		
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=1;
		gbc.gridy=1;
		gbc.weightx=0.5;
		gbc.weighty=0.5;
		gbc.insets = new Insets(0,5,0,5);
		gbc.fill=GridBagConstraints.HORIZONTAL;
		
		
		getContentPane().add(new JLabel("Background Intensity (Blank for auto scale)"),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		
		minIntensity = new JTextField("",5);
		if (prefs.useAutoScale() == false) {
			minIntensity.setText(""+prefs.minIntensity());
		}
		getContentPane().add(minIntensity,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;
		gbc.weightx=0.5;
		
		getContentPane().add(new JLabel("Minimum Ratio"),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		
		minRatio = new JTextField(""+prefs.minRatio(),5);
		getContentPane().add(minRatio,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;
		gbc.weightx=0.5;
		
		getContentPane().add(new JLabel("Maximum Ratio"),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		
		maxRatio = new JTextField(""+prefs.maxRatio(),5);
		getContentPane().add(maxRatio,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;
		gbc.weightx=0.5;

		getContentPane().add(new JLabel("Smoothing Level (0-5)"),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		
		smoothing = new JTextField(""+prefs.smoothing(),5);
		getContentPane().add(smoothing,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;
		gbc.weightx=0.5;

		getContentPane().add(new JLabel("Brightness of FRET image (%)"),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		
		brightness = new JTextField(""+prefs.brightness(),5);
		getContentPane().add(brightness,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;
		gbc.weightx=0.5;

		getContentPane().add(new JLabel("Overlay pixels outside ratio range"),gbc);
		
		gbc.gridx=2;
		gbc.weightx=0.1;
		
		overlay = new JCheckBox();
		overlay.setSelected(prefs.addOverlay());
		getContentPane().add(overlay,gbc);
		
		gbc.gridx=1;
		gbc.gridy++;
		gbc.weightx=0.5;
		gbc.weighty=0.01;
		gbc.gridwidth=2;
		
		JPanel buttonPanel = new JPanel();
		
		JButton closeButton = new JButton("Close");
		closeButton.setActionCommand("close");
		closeButton.addActionListener(this);
		buttonPanel.add(closeButton);
		
		JButton saveButton = new JButton("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		buttonPanel.add(saveButton);
		
		getContentPane().add(buttonPanel,gbc);
		
		setSize(300,300);
		setLocationRelativeTo(application);
		setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {

		if (e.getActionCommand().equals("close")) {
			setVisible(false);
			dispose();
		}
		else if (e.getActionCommand().equals("save")) {
			savePreferences();
		}
	}
	
	private void savePreferences () {
		
		int minIntensity;
		double minRatio;
		double maxRatio;
		int smoothing;
		int brightness;
		
		try { // Outer try catches internal checks on ranges
			try {
				if (this.minIntensity.getText().length() == 0) {
					minIntensity = -1;
				}
				else {
					minIntensity = Integer.parseInt(this.minIntensity.getText());
					if (minIntensity <0) throw new Exception("Min Intensity must be positive");
				}
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Min Intensity must be an integer","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
	
		
			try {
				minRatio = Double.parseDouble(this.minRatio.getText());
				if (minRatio <0) throw new Exception("Min Ratio must be >0");
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Min Ratio must be a number","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
	
	
			try {
				maxRatio = Double.parseDouble(this.maxRatio.getText());
				if (maxRatio <0) throw new Exception("Max Ratio must be >0 ("+maxRatio+")");
				if (maxRatio <= minRatio) throw new Exception("Max Ratio must be greater than Min Ratio");
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Max Ratio must be a number","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
	
	
			try {
				smoothing = Integer.parseInt(this.smoothing.getText());
				if (smoothing <0 || smoothing > 5) throw new Exception("Smoothing must be in the range 0-5");
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Smoothing must be an integer","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			try {
				brightness = Integer.parseInt(this.brightness.getText());
				if (brightness <0 || brightness > 10000) throw new Exception("Brightness must be in the range 1-10000");
			}
			catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(this, "Brightness must be an integer","Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		catch (Exception e) {
			JOptionPane.showMessageDialog(this, e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		

		// OK, we can update the preferences
		FretSawPreferences prefs = FretSawPreferences.getInstance();
		if (minIntensity < 0) {
			prefs.setUseAutoScale(true);
		}
		else {
			prefs.setUseAutoScale(false);
			prefs.setMinIntensity(minIntensity);
		}
		prefs.setRatioRange(minRatio, maxRatio);
		prefs.setSmoothing(smoothing);
		prefs.setBrightness(brightness);
		prefs.setAddOverlay(overlay.isSelected());
		
		setVisible(false);
		application.updateImage();
		dispose();
		
	
	}
	
}
